import subprocess
import sys
import pytest
import uuid

def test_clone_help(setup, script_runner):
    # Test display help message
    ret = script_runner.run(pytest.aoa_script, 'clone', '-h')
    assert ret.stdout.startswith('usage: aoa')
    assert ' clone [-h] [-id PROJECT_ID] [-p PATH] [--debug]\n\noptional arguments:\n' in ret.stdout
    assert ret.stderr == ''
    assert ret.success
    ret = script_runner.run(pytest.aoa_script, 'clone', '--help')
    assert ret.stdout.startswith('usage: aoa')
    assert ' clone [-h] [-id PROJECT_ID] [-p PATH] [--debug]\n\noptional arguments:\n' in ret.stdout
    assert ret.stderr == ''
    assert ret.success


@pytest.mark.parametrize("command,params,input,want", [
    pytest.param("clone", [],
                 b"0\n", {
                     "returncode": 1,
                     "stdout": b"already exists and is not an empty directory.",
                     "stderr": b"",
                 }, id="Clone model with no path"),
    pytest.param("clone", ['--path', '/tempita'],
                 b"0\n", {
                     "returncode": 1,
                     "stdout": b" could not create work tree dir '/tempita'",
                     "stderr": b"",
                 }, id="Clone model with wrong path"),
    # pytest.param("clone", ['--path', '/tmp/aoacli/' + str(uuid.uuid4()), '--test'],
    #              b"0\n", {
    #                  "returncode": 0,
    #                  "stdout": b"Project cloned at",
    #                  "stderr": b"",
    #              }, id="Clone model with path"),
    pytest.param("clone", ['-error'],
                 b"", {
                     "returncode": 2,
                     "stdout": b"",
                     "stderr": b" error: unrecognized arguments: -error",
                 }, id="Configure client error"),
])
def test_clone(command, params, input, want):
    completed_process = run_clone(command, params, input)
    got = {
        "returncode": completed_process.returncode,
        "stdout": completed_process.stdout,
        "stderr": completed_process.stderr,
    }
    assert want['stderr'] in got['stderr']
    assert want['stdout'] in got['stdout']
    assert got['returncode'] == want['returncode']


def run_clone(command, params, input):
    from subprocess import PIPE
    return subprocess.run(
        [sys.executable, pytest.aoa_script, command, *params, "-cwd={}".format(pytest.temp_dir)],
        input=input,
        stdout=PIPE,
        stderr=PIPE
    )



import pytest
import docker
import time
import requests
import logging
import os
from pathlib import Path
from shutil import copyfile

logger = logging.getLogger()

client = docker.from_env()
container_name = "test-aoa-img"
max_retries = 10


def pytest_addoption(parser):
    parser.addoption("--img", action="store", help="specify image name")
    parser.addoption("--port", action="store", help="specify port number")
    parser.addoption("--skip-docker", action="store_true", help="don't start the docker container")


def check_aoa_up(aoa_url, auth):
    retries = 0
    while retries < max_retries:
        try:
            resp = requests.get(
                url=aoa_url + "/admin/health",
                auth=auth
            )
            return resp.status_code
        except:
            logger.info('AOA test container... waiting')
            retries += 1
            time.sleep(5)
    assert (5 == 4), 'AOA test container... unable to access'


@pytest.fixture(scope="session")
def setup(request, tmpdir_factory):
    skip_docker = request.config.getoption("--skip-docker", False)

    if not skip_docker:
        img_name = request.config.getoption("--img") if request.config.getoption("--img") is not None \
    else "thinkbiganalytics/aoa-core:master"
        port_number = request.config.getoption("--port") if request.config.getoption("--port") is not None \
            else 8080

        logger.info("AOA test container using image {} and port {}".format(img_name, port_number))
        logger.info("AOA test container... starting")
        client.containers.run(image=img_name,
                              name=container_name,
                              auto_remove=True,
                              privileged=True,
                              ports={
                                  '8080/tcp': port_number,
                                  '9000/tcp': 9000,
                                  '61613/tcp': 61613
                              },
                              volumes={
                                  '/var/run/docker.sock': {
                                      'bind': '/var/run/docker.sock'
                                  }
                              },
                              detach=True)

        def fin():
            logger.info('AOA test container... removing')
            client.containers.list(all=True, filters={
                "name": container_name
            })[0].remove(force=True)
            logger.info('AOA test container... removed')

        request.addfinalizer(fin)

    from aoa.api_client import AoaClient
    pytest.aoa_client = AoaClient(aoa_url="http://localhost:8080",
                                  auth_mode="basic",
                                  auth_user="admin",
                                  auth_pass="admin")
    pytest.aoa_client.set_project_id("23e1df4b-b630-47a1-ab80-7ad5385fcd8d")
    pytest.module_dir = os.path.join(os.path.dirname(__file__).replace('/tests', ''))
    pytest.aoa_script = os.path.join(pytest.module_dir, 'aoacli.py')
    copyfile(os.path.join(pytest.module_dir, 'scripts/aoa'), pytest.aoa_script)
    os.chmod(pytest.aoa_script, 0o755)

    check_aoa_up(pytest.aoa_client.aoa_url, pytest.aoa_client.auth)
    logger.info('AOA test container... started')

    temp_dir = tmpdir_factory.mktemp("aoa_cli_dir")
    model_temp_dir = os.path.join(str(temp_dir), "model_definitions")
    pytest.temp_dir = temp_dir
    pytest.model_temp_dir = model_temp_dir
    pytest.getbasetemp = tmpdir_factory.getbasetemp
    pytest.command_name = 'aoa'
    logger.debug("AOA testing directory structure created at {}".format(temp_dir))
    logger.info("AOA testing model directory structure created at {}".format(model_temp_dir))

    def cleanup():
        file_to_remove = Path(pytest.aoa_script)
        file_to_remove.unlink()

    request.addfinalizer(cleanup)



import subprocess
import sys
import pytest


def test_run_help(setup, script_runner):
    # Test display help message
    ret = script_runner.run(pytest.aoa_script, 'run', '-h')
    assert ret.stdout.startswith('usage: aoa')
    assert ' run [-h] [-id MODEL_ID] [-m MODE] [-d DATASET_ID]\n' in ret.stdout
    assert ret.stderr == ''
    assert ret.success
    ret = script_runner.run(pytest.aoa_script, 'run', '--help')
    assert ret.stdout.startswith('usage: aoa')
    assert ' run [-h] [-id MODEL_ID] [-m MODE] [-d DATASET_ID]\n' in ret.stdout
    assert ret.stderr == ''
    assert ret.success


@pytest.mark.parametrize("command,params,input,want", [
    pytest.param("run", [],
                 b"0\n0\n0\n", {
                     "returncode": 0,
                     "stdout": b"Loading and executing model code\nStarting training...\nFinished training\nSaved trained model\nArtefacts can be found in:",
                     "stderr": b"",
                 }, id="Train model with no parameters"),
    # pytest.param("run", ['-id', '03c9a01f-bd46-4e7c-9a60-4282039094e6', '-m', 'train', '-d', '11e1df4b-b630-47a1-ab80-7ad5385fcd8c'],
    #              b"", {
    #                  "returncode": 1,
    #                  "stdout": b"Loading and executing model code\nStarting training...\nFinished training\nSaved trained model\nArtefacts can be found in:",
    #                  "stderr": b"",
    #              }, id="Train model with parameters"),
    pytest.param("run", [],
                 b"0\n1\n0\n", {
                     "returncode": 0,
                     "stdout": b"Artefacts can be found in:",
                     "stderr": b"",
                 }, id="Evaluate model with no parameters"),
    # pytest.param("run", ['-id', '03c9a01f-bd46-4e7c-9a60-4282039094e6', '-m', 'evaluate', '-d', '11e1df4b-b630-47a1-ab80-7ad5385fcd8c'],
    #              b"", {
    #                  "returncode": 1,
    #                  "stdout": b"Evaluation metrics can be found in:",
    #                  "stderr": b"",
    #              }, id="Evaluate model with parameters"),
    pytest.param("run", ['-error'],
                 b"", {
                     "returncode": 2,
                     "stdout": b"",
                     "stderr": b" error: unrecognized arguments: -error",
                 }, id="Configure client error"),
])
def test_run(command, params, input, want):
    completed_process = run_run(command, params, input)
    got = {
        "returncode": completed_process.returncode,
        "stdout": completed_process.stdout,
        "stderr": completed_process.stderr,
    }
    assert want['stderr'] in got['stderr']
    assert want['stdout'] in got['stdout']
    assert got['returncode'] == want['returncode']


def run_run(command, params, input):
    from subprocess import PIPE
    return subprocess.run(
        [sys.executable, pytest.aoa_script, command, *params, "-cwd={}".format(pytest.temp_dir)],
        input=input,
        stdout=PIPE,
        stderr=PIPE
    )



import pytest
import uuid
from aoa.api.dataset_template_api import DatasetTemplateApi


def _skip_generated_fields(dataset_template):
    keys_to_remove = ('id', 'ownerId', 'createdAt', '_links', 'projectId')
    for k in keys_to_remove:
        dataset_template.pop(k, None)
    return dataset_template


def test_dataset_template(setup):
    dataset_template_api = DatasetTemplateApi(pytest.aoa_client)
    try:
        before_count = len(dataset_template_api.find_all()['_embedded']['datasetTemplates'])
    except Exception as ex:
        # demo container is initialized without any dataset template
        before_count = 0

    dataset_template_definition = {
        "name": "aoa python client dataset template",
        "description": "adding sample dataset template",
        "templateBody": {
            "basic": {
                "hostname": "mydatabase.com"
            },
            "train": {
                "query": "SELECT * FROM {{table}}",
                "metrics_table": "MY_METRICS_TABLE"
            },
            "evaluate": {
                "query": "SELECT * FROM {{table}}",
                "predictions_table": "MY_PREDICTIONS_TABLE"
            }
        },
        "parameters": {
            "table": "DEFAULT_TABLE"
        }
    }

    dataset_template_rtn = dataset_template_api.save(dataset_template_definition)

    after_count = len(dataset_template_api.find_all()['_embedded']['datasetTemplates'])
    assert ( before_count + 1 == after_count), "save dataset template...failed"

    dataset_template_rtn_by_id = dataset_template_api.find_by_id(dataset_template_id=dataset_template_rtn['id'])
    assert (_skip_generated_fields(dataset_template_rtn_by_id) == dataset_template_definition), "validate find dataset template by id...failed"

    dataset_template_by_name = dataset_template_api.find_by_name(dataset_template_name=dataset_template_definition["name"])
    assert (_skip_generated_fields(dataset_template_by_name) == dataset_template_definition), "validate find dataset template by name...failed"

    dataset_template_rtn_by_id = dataset_template_api.find_by_id(dataset_template_id=str(uuid.uuid4()))
    assert (dataset_template_rtn_by_id is None), "validate missing resouce... failed"

    template_request = {
        "datasetTemplateId": dataset_template_rtn['id'],
        "scope": "train",
        "parameterOverrides": {
            "table": "MY_TRAIN_TABLE"
        }
    }

    rendered_template = dataset_template_api.apply_template(template_request=template_request)
    assert (rendered_template['query'] == "SELECT * FROM MY_TRAIN_TABLE"), "validate apply template with scope -- field: 'query'...failed"
    assert (rendered_template['metrics_table'] == "MY_METRICS_TABLE"), "validate apply template with scope -- field: 'metrics_table'...failed"
    assert (('predictions_table' in rendered_template) == False), "validate apply template with scope -- field: 'predictions_table'...failed"


from __future__ import absolute_import
from typing import Dict

from aoa.api.base_api import BaseApi


class DatasetTemplateApi(BaseApi):
    path = "/api/datasetTemplates/"

    def find_all(self, projection: str = None, page: int = None, size: int = None, sort: str = None):
        """
        returns all dataset templates of a project

        Parameters:
           projection (str): projection type
           page (int): page number
           size (int): number of records in a page
           sort (str): column name and sorting order
           e.g. name?asc: sort name in ascending order, name?desc: sort name in descending order

        Returns:
            (dict): all dataset templates
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection', 'sort', 'page', 'size', 'sort']
        query_vals = [projection, sort, page, size, sort]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path,
            header_params,
            query_params)

    def find_by_id(self, dataset_template_id: str, projection: str = None):
        """
        returns a dataset of a project

        Parameters:
           dataset_template_id (str): dataset template id(uuid) to find
           projection (str): projection type

        Returns:
            (dict): dataset template
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['projection']
        query_vals = [projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + dataset_template_id,
            header_params,
            query_params)

    def find_by_name(self, dataset_template_name: str, projection: str = None):
        """
        returns a dataset template of a project by name

        Parameters:
           dataset_template_name (str): dataset name(string) to find
           projection (str): projection type

        Returns:
            (dict): dataset template
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        query_vars = ['name', 'projection']
        query_vals = [dataset_template_name, projection]
        query_params = self.generate_params(query_vars, query_vals)

        return self.aoa_client.get_request(
            self.path + "search/findByName",
            header_params,
            query_params)

    def save(self, dataset_template: Dict[str, str]):
        """
        register a dataset template

        Parameters:
           dataset template (dict): dataset template to register

        Returns:
            (dict): dataset template
        """
        header_vars = ['AOA-Project-ID', 'Accept']
        header_vals = [
            self.aoa_client.project_id,
            self.aoa_client.select_header_accept([
                'application/json',
                'application/hal+json',
                'text/uri-list',
                'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        self.required_params(['name', 'templateBody', 'parameters'], dataset_template)

        query_params = {}

        return self.aoa_client.post_request(
            self.path,
            header_params,
            query_params,
            dataset_template)

    def apply_template(self, template_request: Dict[str, str]):
        """
        render a dataset template with parameter values

        Parameters:
           template_request (dict): request containing the 'datasetTemplateId'(uuid)
           and 'parameterOverrides'(dict) with the values to render the template

        Returns:
            (dict): rendered dataset template
        """
        header_vars = ['AOA-Project-ID', 'Content-Type', 'Accept']
        header_vals = [
        self.aoa_client.project_id,
        'application/json',
        self.aoa_client.select_header_accept([
            'application/json',
            'application/hal+json',
            'text/uri-list',
            'application/x-spring-data-compact+json'])]
        header_params = self.generate_params(header_vars, header_vals)

        self.required_params(['datasetTemplateId', 'parameterOverrides'], template_request)

        query_params = {}

        return self.aoa_client.post_request(
            self.path + "applyTemplate",
            header_params,
            query_params,
            template_request)



import pytest


def test_help(setup, script_runner):
    # Test display help message
    ret = script_runner.run(pytest.aoa_script, '-h')
    assert ret.stdout.startswith('usage: aoa')
    assert ' [-h] [--debug] [--version]\n' in ret.stdout
    assert ret.stderr == ''
    # Test display help message
    ret = script_runner.run(pytest.aoa_script, '--help')
    assert ret.stdout.startswith('usage: aoa')
    assert ' [-h] [--debug] [--version]\n' in ret.stdout
    assert ret.stderr == ''
    assert ret.success
    assert ret.success


def test_version(setup, script_runner):
    # Test display version
    from aoa import __version__
    ret = script_runner.run(pytest.aoa_script, '--version')
    assert ret.stdout =="\n{}\n".format(__version__)
    assert ret.stderr == ''
    assert ret.success



import pytest

from aoa.api.job_api import JobApi
from aoa.api.trained_model_api import TrainedModelApi

max_retries = 10


@pytest.mark.xfail
def test_model_lifecycle(setup):
    trained_model_api = TrainedModelApi(pytest.aoa_client)
    job_api = JobApi(pytest.aoa_client)

    # Train
    training_request = dict()
    model_id = '03c9a01f-bd46-4e7c-9a60-4282039094e6'
    training_request['modelId'] = model_id
    training_request['datasetId'] = '11e1df4b-b630-47a1-ab80-7ad5385fcd8c'

    job_rtn = trained_model_api.train(training_request)
    job_id = job_rtn['id']
    job_rtn_by_id = job_api.find_by_id(job_id=job_id)

    assert (job_rtn_by_id['metadata']['trainingRequest'] == training_request), "validate training request...failed"

    job_api.wait(job_id)

    trained_model_id = job_rtn_by_id['metadata']['trainedModel']['id']
    trained_model_by_id = trained_model_api.find_by_id(trained_model_id=trained_model_id,
                                                       projection="expandTrainedModel")
    assert (trained_model_by_id['modelId'] == training_request['modelId'] and
            trained_model_by_id['dataset']['id'] == training_request['datasetId']), "train model...failed"

    # Evaluate
    evaluation_request = dict()
    evaluation_request['datasetId'] = '11e1df4b-b630-47a1-ab80-7ad5385fcd8c'

    job_eval_rtn = trained_model_api.evaluate(trained_model_id, evaluation_request)
    job_eval_id = job_eval_rtn['id']
    job_eval_rtn_by_id = job_api.find_by_id(job_id=job_eval_id)

    assert (job_eval_rtn_by_id['metadata']['evaluationRequest'] == evaluation_request), \
        "validate evaluate request...failed"

    job_api.wait(job_eval_id)

    trained_model_eval_by_id = trained_model_api.find_by_id(trained_model_id=trained_model_id,
                                                            projection="expandTrainedModel")
    assert (trained_model_eval_by_id['dataset']['id'] == evaluation_request['datasetId']), "evaluate model...failed"

    # Deploy
    transition_request = dict()
    transition_request['status'] = 'DEPLOYED'

    job_deployed = trained_model_api.transition(trained_model_id, transition_request)
    job_api.wait(job_deployed['id'])
    assert('type' in job_deployed and job_deployed['type'] == 'DEPLOYMENT'), "Deployment job has not been scheduled"

    deployed_status = trained_model_api.find_by_model_id_and_status(model_id=model_id, status="DEPLOYED")
    assert (1 == deployed_status['page']['totalElements']), "Model has not been deployed"

    # Retire
    transition_request = dict()
    transition_request['status'] = 'RETIRED'

    job_retired = trained_model_api.transition(trained_model_id, transition_request)
    job_api.wait(job_retired['id'])
    assert('type' in job_retired and job_retired['type'] == 'RETIREMENT'), "Retirement job has not been scheduled"

    retired_status = trained_model_api.find_by_model_id_and_status(model_id=model_id, status="RETIRED")
    assert (1 == retired_status['page']['totalElements']), "Model has not been retired"



import pytest
import tempfile

from aoa.api.trained_model_api import TrainedModelApi
from aoa.api.model_api import ModelApi


def test_byom_models(setup):
    model_api = ModelApi(pytest.aoa_client)
    trained_model_api = TrainedModelApi(pytest.aoa_client)

    model = {
        "name": "My PMML",
        "description": "Test PMML Model",

        "externalModelAttributes": {
            "format": "PMML"
        }
    }

    model = model_api.save(model)

    trained_model = {
        "modelId": model["id"],
        "metadata": {
            "configuration": {
                "key1": "val1"
            }
        }
    }

    with tempfile.NamedTemporaryFile(delete=False) as fp:
        fp.write(b'Hello world!')

    trained_model_api.save(trained_model, [fp.name])




import subprocess
import sys
import pytest


def test_init_repo(setup, script_runner):
    # Test display help message
    ret = script_runner.run(pytest.aoa_script, 'init', '-h')
    assert ret.stdout.startswith('usage: aoa')
    assert ' init [-h] [--debug]\n\noptional arguments:\n' in ret.stdout
    assert ret.stderr == ''
    assert ret.success
    ret = script_runner.run(pytest.aoa_script, 'init', '--help')
    assert ret.stdout.startswith('usage: aoa')
    assert ' init [-h] [--debug]\n\noptional arguments:\n' in ret.stdout
    assert ret.stderr == ''
    assert ret.success


@pytest.mark.parametrize("command,params,input,want", [
    pytest.param("init", ['-h'],
                 b"", {
                     "returncode": 0,
                     "stdout": b" init [-h] [--debug]\n\noptional arguments:\n",
                     "stderr": b"",
                 }, id="Display help message with -h"),
    pytest.param("init", ['--help'],
                 b"", {
                     "returncode": 0,
                     "stdout": b" init [-h] [--debug]\n\noptional arguments:\n",
                     "stderr": b"",
                 }, id="Display help message with --help"),
    pytest.param("configure", ['--test'],
                 b"http://localhost:8080\n0\nadmin\nadmin\n", {
                     "returncode": 0,
                     "stdout": b"New configuration saved at",
                     "stderr": b"",
                 }, id="Configure client"),
    pytest.param("init", ['--debug'],
                 b"0\n", {
                     "returncode": 0,
                     "stdout": b"Creating model directory\nCreating model definitions\nmodel directory initialized at",
                     "stderr": b"",
                 }, id="Initialize project structure"),
    pytest.param("init", ['-error'],
                 b"", {
                     "returncode": 2,
                     "stdout": b"",
                     "stderr": b" error: unrecognized arguments: -error",
                 }, id="Init error"),
])
def test_init(command, params, input, want):
    completed_process = run_init(command, params, input)
    got = {
        "returncode": completed_process.returncode,
        "stdout": completed_process.stdout,
        "stderr": completed_process.stderr,
    }
    assert want['stderr'] in got['stderr']
    assert want['stdout'] in got['stdout']
    assert got['returncode'] == want['returncode']


def run_init(command, params, input):
    from subprocess import PIPE
    return subprocess.run(
        [sys.executable, pytest.aoa_script, command, *params, "-cwd={}".format(pytest.temp_dir)],
        input=input,
        stdout=PIPE,
        stderr=PIPE
    )
